// *********************************************************************
//
//
// Bitec DisplayPort IP Core
// 
//
// All rights reserved. Property of Bitec.
// Restricted rights to use, duplicate or disclose this code are
// granted through contract.
// 
// (C) Copyright Bitec 2010,2011,2012,2013,2014
//     All rights reserved
//
// *********************************************************************
// Author         : $Author: Marco $ @ bitec-dsp.com
// Department     : 
// Date           : $Date: 2014-11-21 15:32:16 +0200 (Fri, 21 Nov 2014) $
// Revision       : $Revision: 1501 $
// URL            : $URL: svn://nas-bitec-fi/dp/trunk/demo/bitec_reconfig_alt_av.v $
// *********************************************************************
// Description
// 
// This module implements Altera RX/TX transceivers reconfiguration
// for Altera Arria 10 GX:
// - single 135MHz clock input for 1620,2700,5400 and 8100 Mbps
// - TX bonded
// - Logical channels ordering:
//       index                         description
//   0 to RX_LANES-1                   RX channels
//   RX_LANES to RX_LANES+TX_LANES-1   TX channels
//   RX_LANES+TX_LANES                 TX (single) PLL
//
// *********************************************************************

// synthesis translate_off
`timescale 1ns / 1ns
// synthesis translate_on
`default_nettype none

module bitec_reconfig_alt_a10 
#(
  parameter [3:0] TX_LANES = 4,
  parameter [3:0] RX_LANES = 4
)
(
  input  wire clk,                        // The same clock driving the reconfig controllers
  input  wire reset,                      // The same reset driving the reconfig controllers

  input  wire [7:0] rx_link_rate,         // Link rate in multiples of 270 Mbps
  input  wire rx_link_rate_strobe,        // Assert for at least 1 clk cycle when a new rx_link_rate must be used
  output wire rx_xcvr_busy,               // Asserted during RX reconfig and calibration time

  input  wire [7:0] tx_link_rate,         // Link rate in multiples of 270 Mbps
  input  wire [(TX_LANES*2)-1:0] tx_vod,  // Voltage swing level, 0 to 3
  input  wire [(TX_LANES*2)-1:0] tx_emp,  // Pre-emphasis level, 0 to 3
  input  wire tx_link_rate_strobe,        // Assert for at least 1 clk cycle when a new tx_link_rate must be used
  input  wire tx_vodemp_strobe,           // Assert for at least 1 clk cycle when new VOD/EMP values must be used
  output wire tx_xcvr_busy,               // Asserted during TX reconfig and calibration time
  output wire tx_xcvr_reset,              // Asserted when the TX XCVR must be reset

  // Reconfig controller interface
  output wire [1:0] mgmt_chnum,
  output wire [9:0] mgmt_address,
  output wire [31:0] mgmt_writedata,
  input  wire [31:0] mgmt_readdata,
  output wire mgmt_write,
  output wire mgmt_read,
  input  wire mgmt_waitrequest,

  input  wire rx_cal_busy,
  input  wire tx_cal_busy
);

// main FSM states
localparam  FSM_IDLE                    = 4'd0,
            FSM_START_RX_LINKRATE       = 4'd1,
            FSM_START_TX_LINKRATE       = 4'd2,
            FSM_START_TX_ANALOG         = 4'd3,
            FSM_FEAT_RECONFIG           = 4'd4,
            FSM_WAIT_FOR_BUSY_LOW       = 4'd5,
            FSM_NEXT_RX_LRATE_FEATURE   = 4'd6,
            FSM_NEXT_TX_LRATE_FEATURE   = 4'd7,
            FSM_NEXT_TX_ANALOG_FEATURE  = 4'd8,
            FSM_END_RECONFIG            = 4'd9,
            FSM_NEXT_LANE               = 4'd10,
            FSM_END                     = 4'd11;

// Feature index
localparam  FEAT_RX_REFCLK1 = 4'd0,
            FEAT_RX_REFCLK2 = 4'd1,
            FEAT_RX_REFCLK3 = 4'd2,
            FEAT_TX_VOD     = 4'd3,
            FEAT_TX_EMP1    = 4'd4,
            FEAT_TX_EMP2    = 4'd5,
            FEAT_TX_EMP3    = 4'd6,
            FEAT_TX_EMP4    = 4'd7,
            FEAT_TX_REFCLK1 = 4'd8,
            FEAT_TX_REFCLK2 = 4'd9,
            FEAT_TX_REFCLK3 = 4'd10;

// XCVR Reconfiguration controller register addresses
localparam  ADDR_PRE_EMP_SWITCHING_CTRL_1ST_POST_TAP    = 10'h105,
            ADDR_PRE_EMP_SWITCHING_CTRL_2ND_POST_TAP    = 10'h106,
            ADDR_PRE_EMP_SWITCHING_CTRL_PRE_TAP_1T      = 10'h107,
            ADDR_PRE_EMP_SWITCHING_CTRL_PRE_TAP_2T      = 10'h108,
            ADDR_VOD_OUTPUT_SWING_CTRL                  = 10'h109;

// XCVR Reconfiguration controller register masks
localparam  MASK_PRE_EMP_SWITCHING_CTRL_1ST_POST_TAP    = 32'h0000_005f,
            MASK_PRE_EMP_SWITCHING_CTRL_2ND_POST_TAP    = 32'h0000_002f,
            MASK_PRE_EMP_SWITCHING_CTRL_PRE_TAP_1T      = 32'h0000_003f,
            MASK_PRE_EMP_SWITCHING_CTRL_PRE_TAP_2T      = 32'h0000_0017,
            MASK_VOD_OUTPUT_SWING_CTRL                  = 32'h0000_001f;



reg [3:0] fsm_state; // Main FSM state
reg [3:0] feature_idx; // Feature index

// VOD & EMP mapped to the TX XCVR
wire [TX_LANES*5-1:0] vod_mapped;
wire [TX_LANES*6-1:0] emp_mapped;   

// Generate mapping tables for each lane
generate 
begin      
  genvar tx_lane;
  for (tx_lane=0; tx_lane < TX_LANES; tx_lane = tx_lane + 1) 
  begin:lane
    dp_analog_mappings dp_analog_mappings_i
    (
      .vod                  (tx_vod[(tx_lane*2) +:2]), 
      .pree                 (tx_emp[(tx_lane*2) +:2]),
      .out_vod              (vod_mapped[(tx_lane*5) +:5]), 
      .out_pree_post_tap1   (emp_mapped[(tx_lane*6) +:6])
    );
  end // for
end // generate
endgenerate

//-------------------
// Changes detection
//-------------------

reg [2:0] rx_link_rate_strobe_d,tx_link_rate_strobe_d,tx_vodemp_strobe_d;
reg rx_new_linkrate; // Asserted when RX linkrate changes are detected
reg tx_new_linkrate; // Asserted when TX linkrate changes are detected
reg tx_new_analog;   // Asserted when TX VOD/EMP changes are detected

always @ (posedge clk or posedge reset) 
begin
  if(reset) 
  begin
    rx_link_rate_strobe_d <= 3'h0;
    tx_link_rate_strobe_d <= 3'h0;
    tx_vodemp_strobe_d <= 3'h0;
    rx_new_linkrate <= 1'b0;
    tx_new_linkrate <= 1'b0;
    tx_new_analog <= 1'b0;
  end
  else
  begin
    rx_link_rate_strobe_d <= {rx_link_rate_strobe_d[1:0],rx_link_rate_strobe};
    tx_link_rate_strobe_d <= {tx_link_rate_strobe_d[1:0],tx_link_rate_strobe};
    tx_vodemp_strobe_d <= {tx_vodemp_strobe_d[1:0],tx_vodemp_strobe};
    
    rx_new_linkrate <= rx_new_linkrate ? ~(fsm_state == FSM_START_RX_LINKRATE) : (~rx_link_rate_strobe_d[2] & rx_link_rate_strobe_d[1]);
    tx_new_linkrate <= tx_new_linkrate ? ~(fsm_state == FSM_START_TX_LINKRATE) : (~tx_link_rate_strobe_d[2] & tx_link_rate_strobe_d[1]);
    tx_new_analog <= tx_new_analog ? ~(fsm_state == FSM_START_TX_ANALOG) : (~tx_vodemp_strobe_d[2] & tx_vodemp_strobe_d[1]);
  end
end

//----------
// Main FSM
//----------

reg [TX_LANES*5-1:0] vod_mem;
reg [TX_LANES*6-1:0] emp_mem;
reg [1:0] tx_link_rate_mem;
reg [1:0] rx_link_rate_mem;
reg [1:0] lane_idx; // Configured lane index (0-3)
reg [2:0] write_cnt; // Write operations counter

reg rcnf_reconfig;      // Asserted to start a single item reconfig with the values defined below
reg [9:0] rcnf_address; // Reconfiguration address
reg [31:0] rcnf_data;   // Reconfiguration data value
reg [31:0] rcnf_mask;   // Reconfiguration data mask
reg rcnf_begin;
reg rcnf_end;
wire rcnf_busy;

reg rx_lrate_busy;  // Asserted when RX linkrate changes are being performed
reg tx_lrate_busy;  // Asserted when TX linkrate changes are being performed
reg tx_analog_busy; // Asserted when TX analog changes are being performed

assign rx_xcvr_busy = rx_lrate_busy | rx_cal_busy;
assign tx_xcvr_busy = tx_lrate_busy | tx_cal_busy | tx_analog_busy;
assign tx_xcvr_reset = tx_lrate_busy | (tx_analog_busy & (fsm_state == FSM_END));

always @ (posedge clk or posedge reset) 
begin
  if(reset) 
  begin
    fsm_state <= FSM_IDLE;
    vod_mem <= 0;
    emp_mem <= 0;
    tx_link_rate_mem <= 2'h0;
    rx_link_rate_mem <= 2'h0;
    
    rcnf_begin <= 1'b0;
    rcnf_end <= 1'b0;
    rcnf_reconfig <= 1'b0;
    rcnf_address <= 10'h0;
    rcnf_data <= 32'h0;
    rcnf_mask <= 32'h0;

    feature_idx <= FEAT_RX_REFCLK1;
    write_cnt <= 3'h0;
    lane_idx <= 2'h0;
    rx_lrate_busy <= 1'b0;
    tx_lrate_busy <= 1'b0;
    tx_analog_busy <= 1'b0;
  end
  else
  begin
  
    rcnf_begin <= 1'b0;
    rcnf_end <= 1'b0;
    rcnf_reconfig <= 1'b0;

    case(fsm_state)
    
      FSM_IDLE: 
      begin
        write_cnt <= 3'h0;
        lane_idx <= 2'h0;

        if(~rx_xcvr_busy & ~tx_xcvr_busy)
        begin
          // Start a reconfig if either RX or TX requests it
          // (RX has precedence, do not serve RX and TX at the same time)
          if(rx_new_linkrate)
          begin
            rx_lrate_busy  <= 1'b1;
            rcnf_begin <= 1'b1;
            fsm_state <= FSM_START_RX_LINKRATE;
            if(rx_link_rate == 8'h06)
              rx_link_rate_mem <= 2'b00; // RBR
            else if(rx_link_rate == 8'h0a)
              rx_link_rate_mem <= 2'b01; // HBR
            else if(rx_link_rate == 8'h14)
              rx_link_rate_mem <= 2'b10; // HBR2
            else
            begin
              // Unsupported link rate
              rx_lrate_busy  <= 1'b0;
              fsm_state <= FSM_IDLE;
            end
          end
          else if(tx_new_linkrate)
          begin
            tx_lrate_busy  <= 1'b1;
            rcnf_begin <= 1'b1;
            fsm_state <= FSM_START_TX_LINKRATE;
            if(tx_link_rate == 8'h06)
              tx_link_rate_mem <= 2'b00; // RBR
            else if(tx_link_rate == 8'h0a)
              tx_link_rate_mem <= 2'b01; // HBR
            else if(tx_link_rate == 8'h14)
              tx_link_rate_mem <= 2'b10; // HBR2
            else
            begin
              // Unsupported link rate
              tx_lrate_busy  <= 1'b0;
              fsm_state <= FSM_IDLE;
            end
          end
          else if(tx_new_analog)
          begin
            vod_mem <= vod_mapped;
            emp_mem <= emp_mapped;
            tx_analog_busy  <= 1'b1;
            rcnf_begin <= 1'b1;
            fsm_state <= FSM_START_TX_ANALOG;
          end
        end
      end

      FSM_START_RX_LINKRATE: // Start RX linkrate reconfig
      begin
        feature_idx <= FEAT_RX_REFCLK1; // Start programming from this feature
        if(!rcnf_busy & !rcnf_begin) 
          fsm_state <= FSM_FEAT_RECONFIG;
      end

      FSM_START_TX_LINKRATE: // Start TX linkrate reconfig
      begin
        feature_idx <= FEAT_TX_REFCLK1; // Start programming from this feature
        if(!rcnf_busy & !rcnf_begin) 
          fsm_state <= FSM_FEAT_RECONFIG;
      end
        
      FSM_START_TX_ANALOG: // Start TX analog reconfig
      begin
        feature_idx <= FEAT_TX_VOD; // Start programming from this feature
        if(!rcnf_busy & !rcnf_begin) 
          fsm_state <= FSM_FEAT_RECONFIG;
      end
        
      FSM_FEAT_RECONFIG: // Reconfigure a single feature
      begin
        // Setup the registers feeding the low-level FSM based on feature_idx
        // The order is RX channels, TX Analog parameters, TX channels, TX PLLs

        rcnf_reconfig <= 1'b1;
        write_cnt <= write_cnt + 1'd1;
        fsm_state <= FSM_WAIT_FOR_BUSY_LOW;
        
        case(feature_idx)
          FEAT_RX_REFCLK1: // This uses "RX channel" logical channels
          begin
          end
          FEAT_RX_REFCLK2: // This uses "RX channel" logical channels
          begin
          end
          FEAT_RX_REFCLK3: // This uses "RX channel" logical channels
          begin
          end
          
          FEAT_TX_VOD: // TX analog PMA
          begin
            rcnf_address        <= ADDR_VOD_OUTPUT_SWING_CTRL;
            rcnf_mask           <= MASK_VOD_OUTPUT_SWING_CTRL;
            rcnf_data           <= {27'd0,vod_mem[5*lane_idx +:5]};
          end
          FEAT_TX_EMP1: // TX analog PMA
          begin
            rcnf_address        <= ADDR_PRE_EMP_SWITCHING_CTRL_1ST_POST_TAP;
            rcnf_mask           <= MASK_PRE_EMP_SWITCHING_CTRL_1ST_POST_TAP;
            rcnf_data           <= {25'd0,emp_mem[6*lane_idx+5],1'b0,emp_mem[6*lane_idx +:5]};
          end
          FEAT_TX_EMP2: // TX analog PMA
          begin
            rcnf_address        <= ADDR_PRE_EMP_SWITCHING_CTRL_2ND_POST_TAP;
            rcnf_mask           <= MASK_PRE_EMP_SWITCHING_CTRL_2ND_POST_TAP;
            rcnf_data           <= {26'd0,1'b1,1'b0,4'd0};
          end
          FEAT_TX_EMP3: // TX analog PMA
          begin
            rcnf_address        <= ADDR_PRE_EMP_SWITCHING_CTRL_PRE_TAP_1T;
            rcnf_mask           <= MASK_PRE_EMP_SWITCHING_CTRL_PRE_TAP_1T;
            rcnf_data           <= {26'd0,1'b1,5'd0};
          end
          FEAT_TX_EMP4: // TX analog PMA
          begin
            rcnf_address        <= ADDR_PRE_EMP_SWITCHING_CTRL_PRE_TAP_2T;
            rcnf_mask           <= MASK_PRE_EMP_SWITCHING_CTRL_PRE_TAP_2T;
            rcnf_data           <= {27'd0,1'b1,1'b0,3'd0};
          end

          FEAT_TX_REFCLK1: // This uses "TX channel" logical channels
          begin
          end
          FEAT_TX_REFCLK2: // This uses "TX PLL" logical channels
          begin
          end
          FEAT_TX_REFCLK3: // This uses "TX PLL" logical channels
          begin
          end

        endcase

      end

      FSM_WAIT_FOR_BUSY_LOW:
        if(!rcnf_busy & !rcnf_reconfig) 
        begin
          if(rx_lrate_busy)
            fsm_state <= FSM_NEXT_RX_LRATE_FEATURE;
          else if(tx_lrate_busy)
            fsm_state <= FSM_NEXT_TX_LRATE_FEATURE;
          else
            fsm_state <= FSM_NEXT_TX_ANALOG_FEATURE;
        end
    
      FSM_NEXT_RX_LRATE_FEATURE:
      begin
        if(write_cnt == 3'd3)
        begin
          rcnf_end <= 1'b1;
          fsm_state <= FSM_END_RECONFIG;
        end
        else
        begin
          // Next feature
          feature_idx <= feature_idx + 4'd1;
          fsm_state <= FSM_FEAT_RECONFIG;
        end
      end
      
      FSM_NEXT_TX_LRATE_FEATURE:
      begin
        if(write_cnt == 3'd3) 
        begin
          rcnf_end <= 1'b1;
          fsm_state <= FSM_END_RECONFIG;
        end
        else
        begin
          // Next feature
          feature_idx <= feature_idx + 4'd1;
          fsm_state <= FSM_FEAT_RECONFIG;
        end
      end
     
      FSM_NEXT_TX_ANALOG_FEATURE:
      begin
        if(write_cnt == 3'd5) 
        begin
          rcnf_end <= 1'b1;
          fsm_state <= FSM_END_RECONFIG;
        end
        else
        begin
          // Next feature
          feature_idx <= feature_idx + 4'd1;
          fsm_state <= FSM_FEAT_RECONFIG;
        end
      end
     
      FSM_END_RECONFIG:
        if(!rcnf_busy & !rcnf_end) 
          fsm_state <= FSM_NEXT_LANE;

      FSM_NEXT_LANE: // Reconfigure the features for all the lanes
      begin
        if((rx_lrate_busy  & (lane_idx + 1 < RX_LANES)) |
           (tx_lrate_busy  & (lane_idx + 1 < TX_LANES)) |
           (tx_analog_busy & (lane_idx + 1 < TX_LANES)))
        begin
          // Go to next lane
          lane_idx <= lane_idx + 2'd1;
          write_cnt <= 3'd0;
          rcnf_begin <= 1'b1;

          if(rx_lrate_busy)
            fsm_state <= FSM_START_RX_LINKRATE;
          else if(tx_lrate_busy)
            fsm_state <= FSM_START_TX_LINKRATE;
          else
            fsm_state <= FSM_START_TX_ANALOG;
        end
        else
          fsm_state <= FSM_END;        
      end
      
      FSM_END:
      begin
        rx_lrate_busy <= 1'b0;
        tx_lrate_busy <= 1'b0;
        tx_analog_busy <= 1'b0;
        fsm_state <= FSM_IDLE;
      end
    
    endcase
    
  end // if(reset)
end // always

// Instantiate the Avalon MM Master connected to the Reconfiguration Controller
bitec_reconfig_avalon_mm_master bitec_reconfig_avalon_mm_master_i
(
  .clk          (clk),
  .reset        (reset),

  .rcnf_begin         (rcnf_begin),
  .rcnf_end           (rcnf_end),
  .rcnf_reconfig      (rcnf_reconfig),
  .rcnf_logical_ch    (lane_idx),
  .rcnf_address       (rcnf_address),
  .rcnf_data          (rcnf_data),
  .rcnf_mask          (rcnf_mask),
  .rcnf_busy          (rcnf_busy),

  .mgmt_chnum         (mgmt_chnum),
  .mgmt_address       (mgmt_address), 
  .mgmt_writedata     (mgmt_writedata),    
  .mgmt_readdata      (mgmt_readdata),    
  .mgmt_write         (mgmt_write),      
  .mgmt_read          (mgmt_read),      
  .mgmt_waitrequest   (mgmt_waitrequest)
);


endmodule

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//----------------------------------------------
// Avalon MM Master for driving the Reconfiguration
// (XCVR or ATXPLL) management interface
//----------------------------------------------

module bitec_reconfig_avalon_mm_master 
(

  input wire clk,
  input wire reset,

  input wire rcnf_begin,                  // Assert for 1 clk cycle to start reconfig
  input wire rcnf_end,                    // Assert for 1 clk cycle to end reconfig
  input wire rcnf_reconfig,               // Assert for 1 clk cycle to reconfig an address
  input wire [1:0] rcnf_logical_ch,       // Logical channel number
  input wire [9:0] rcnf_address,          // Logical channel address
  input wire [31:0] rcnf_data,            // Value of the data to write
  input wire [31:0] rcnf_mask,            // Mask for the data to write (bits at 1 get written)
  output wire rcnf_busy,                   // Asserted while reconfiguration is takign place

  // Reconfig management interface
  output reg [1:0] mgmt_chnum,
  output reg [9:0] mgmt_address,
  output reg [31:0] mgmt_writedata,
  input wire [31:0] mgmt_readdata,
  output reg mgmt_write,
  output reg mgmt_read,
  input  wire mgmt_waitrequest
);

// State variables
localparam  FSM_IDLE      = 3'd0,
            FSM_START     = 3'd1,
            FSM_READ      = 3'd2,
            FSM_WRITE     = 3'd3,
            FSM_END       = 3'd4;

reg [2:0] state;

always @ (posedge clk or posedge reset) 
  if(reset) 
  begin
    state <= FSM_IDLE;
    mgmt_chnum  <= 2'd0;
    mgmt_address  <= 10'd0;
    mgmt_write <= 1'b0;
    mgmt_read <= 1'b0;
    mgmt_writedata <= 32'd0;
  end
  else
  begin
  
    mgmt_write <= 1'b0;  
    mgmt_read <= 1'b0;

    case (state)
      FSM_IDLE:
      begin
        if(rcnf_begin)
          state <= FSM_START;
        if(rcnf_reconfig)
          state <= FSM_READ;
        if(rcnf_end)
          state <= FSM_END;
      end
      
      FSM_START: 
        begin
          if(mgmt_write & !mgmt_waitrequest) 
            state <= FSM_IDLE;
          else
          begin
            mgmt_chnum     <= rcnf_logical_ch;
            mgmt_address   <= 32'd0;
            mgmt_writedata <= 32'h0000_0002;  
            mgmt_write     <= 1'b1;  
          end
        end
        
      FSM_READ: 
        begin
          if(mgmt_read & !mgmt_waitrequest) 
            state <= FSM_WRITE;
          else
          begin
            mgmt_address   <= rcnf_address;
            mgmt_read      <= 1'b1;  
          end
        end

      FSM_WRITE:
        begin
          if(mgmt_write & !mgmt_waitrequest) 
            state <= FSM_IDLE;
          else
          begin
            mgmt_writedata  <= (mgmt_readdata & ~rcnf_mask) | rcnf_data;
            mgmt_write      <= 1'b1;  
          end
        end
    
      FSM_END: 
        begin
          if(mgmt_write & !mgmt_waitrequest) 
            state <= FSM_IDLE;
          else
          begin
            mgmt_address   <= 32'd0;
            mgmt_writedata <= 32'h0000_0003;  
            mgmt_write     <= 1'b1;  
          end
        end
      endcase
      
    end

assign rcnf_busy = (state != FSM_IDLE);

endmodule

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//----------------------------------------------
// Map DisplayPort Voltage swing / Pre-emphasis
// to transceiver VOD / EMP settings
// for Arria 10 GX
//
// 1st pre-tap always supposed set to 0
// 2nd pre-tap always supposed set to 0
// 2nd post-tap always supposed set to 0
//----------------------------------------------

module dp_analog_mappings 
(
  input  wire [1:0]   vod,  
  input  wire [1:0]   pree, 
  output reg  [4:0]   out_vod, 
  output reg  [5:0]   out_pree_post_tap1 // bit5 is polarity, 1=neg 2=pos
);

always @(*)
  case (vod)
    2'b00 :  // 400mv
    begin
      case(pree) 
        2'b00 : // (0db)
        begin
          out_vod = 5'd13;     
          out_pree_post_tap1 = {1'b1,5'd0};
        end
        2'b01 : // (3.5db)
        begin
          out_vod = 5'd19;     
          out_pree_post_tap1 = {1'b1,5'd6};
        end
        2'b10 : // (6db)
        begin
          out_vod = 5'd25;     
          out_pree_post_tap1 = {1'b1,5'd12};
        end
        2'b11 : // (9db)
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd19};
        end
      endcase
    end
    2'b01 : // 600mv
    begin
      case(pree) 
        2'b00 : // (0db)
        begin
          out_vod = 5'd19;     
          out_pree_post_tap1 = {1'b1,5'd0};
        end
        2'b01 : // (3.5db)
        begin
          out_vod = 5'd28;     
          out_pree_post_tap1 = {1'b1,5'd9};
        end
        2'b10 : // (6db)
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd14};
        end
        2'b11 : // unused
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd14};
        end
      endcase
    end
    2'b10 : // 800mv
    begin
      case(pree) 
        2'b00 : // (0db)
        begin
          out_vod = 5'd25;     
          out_pree_post_tap1 = {1'b1,5'd0};
        end
        2'b01 : // (3.5db)
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd6}; 
        end
        2'b10 : // unused
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd6}; 
        end
        2'b11 : // unused
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd6}; 
        end
      endcase
    end
    2'b11 : // 1200mv
    begin
      case(pree) 
        2'b00 : // (0db)
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd0};
        end
        2'b01 : // 
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd0}; 
        end
        2'b10 : // unused
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd0}; 
        end
        2'b11 : // unused
        begin
          out_vod = 5'd31;     
          out_pree_post_tap1 = {1'b1,5'd0}; 
        end
      endcase
    end
  endcase

endmodule

// synthesis translate_off

module bitec_reconfig_tb();

localparam TX_LANES=4, RX_LANES=4;

reg clk,reset;
reg rx_link_rate_strobe,tx_link_rate_strobe,tx_vodemp_strobe,tx_hdmi_en;

wire [9:0] mgmt_address;
wire [31:0] mgmt_writedata;
wire [31:0] mgmt_readdata;
wire mgmt_write;
wire mgmt_read;
wire mgmt_waitrequest;

always
begin
  #50 clk <= 0;
  #50 clk <= 1;
end

bitec_reconfig_alt_a10
#(
  .TX_LANES   (TX_LANES),
  .RX_LANES   (RX_LANES)
)
dut
(
  .clk      (clk),                
  .reset    (reset),               

  .rx_link_rate         (2'b01),  
  .rx_link_rate_strobe  (rx_link_rate_strobe),
  .rx_xcvr_busy         (),    

  .tx_link_rate         (8'h14),  
  .tx_vod               (8'b00000000), 
  .tx_emp               (8'b10101010), 
  .tx_link_rate_strobe  (tx_link_rate_strobe),
  .tx_vodemp_strobe     (tx_vodemp_strobe),
  .tx_xcvr_busy         (),    
  .tx_xcvr_reset        (),

  .mgmt_chnum           (),
  .mgmt_address         (mgmt_address),
  .mgmt_writedata       (mgmt_writedata),
  .mgmt_readdata        (mgmt_readdata),
  .mgmt_write           (mgmt_write),
  .mgmt_read            (mgmt_read),
  .mgmt_waitrequest     (mgmt_waitrequest),

  .rx_cal_busy          (0),
  .tx_cal_busy          (0)
);

// Instantiate model of reconfig controller
reconfig_model reconfig_model_i 
(
  .clk              (clk),
  .reset            (reset),
  .mgmt_address     (mgmt_address),    
  .mgmt_writedata   (mgmt_writedata),  
  .mgmt_readdata        (mgmt_readdata),
  .mgmt_write           (mgmt_write),
  .mgmt_read            (mgmt_read),
  .mgmt_waitrequest (mgmt_waitrequest)  
);

initial
begin

  #1
  
  rx_link_rate_strobe = 0;
  tx_link_rate_strobe = 0;
  tx_vodemp_strobe = 0;
  tx_hdmi_en = 0;
  
  reset = 1;
  #100
  reset = 0;

  #500
/*
  tx_link_rate_strobe = 1;
  #100
  tx_link_rate_strobe = 0;
  #12000
  rx_link_rate_strobe = 1;
  #100
  rx_link_rate_strobe = 0;
  #12000
*/
  tx_vodemp_strobe = 1;
  #100
  tx_vodemp_strobe = 0;
  
end

endmodule

// *********************************************************************
// This is a simple behavioral model of the reconfig controller used 
// for simulation testing.
// *********************************************************************
module reconfig_model 
(
  input wire clk,
  input wire reset,

  input wire [9:0] mgmt_address,
  input wire [31:0] mgmt_writedata,
  output reg [31:0] mgmt_readdata,
  input wire mgmt_write,        
  input wire mgmt_read,        
  output wire mgmt_waitrequest
);

// State variables
localparam  IDLE = 0, READ = 1;

reg [1:0] state;
reg [1:0] cnt;
reg ready;
assign mgmt_waitrequest = ~ready & mgmt_read;

always @ (posedge clk or posedge reset) 
  if (reset) 
  begin
    state <= IDLE;
    mgmt_readdata <= 32'd0;
    cnt <= 0;
    ready <= 0;
  end
  else
  begin
  
    ready <= 0;
    case (state)
      IDLE:
        begin
          if (mgmt_write)
            cnt <= cnt + 1;
          if (mgmt_read & ~ready)
          begin
            cnt <= cnt + 1;
            state <= READ;
          end
        end

      READ:
      begin
        cnt <= cnt + 1;
        if(&cnt)
        begin
          ready <= 1;
          mgmt_readdata <= 32'h1111_2222;
          state <= IDLE;
        end
      end

    endcase
    
  end

endmodule

// synthesis translate_on


`default_nettype wire
